import { ValidationOptions } from 'class-validator/types/decorator/ValidationOptions';
import { ValidatorConstraintInterface, ValidationArguments } from 'class-validator';

interface PasswordValidatorOptions {
    minLength?: LengthRule;
    maxLength?: LengthRule;
    hasUpperCase?: HasRule;
    hasLowerCase?: HasRule;
    hasNumbers?: HasRule;
    hasSpecialChars?: HasSpecialCharsRule;
    blacklist?: BlacklistRule;
    confirmPassword?: ConfirmPasswordRule;
}
interface LengthRule {
    message?: string;
    enabled: boolean;
    length?: number;
}
interface BlacklistRule {
    message?: string;
    enabled: boolean;
    values: string[];
}
interface HasSpecialCharsRule {
    message?: string;
    enabled: boolean;
    characters?: string[];
}
interface HasRule {
    message?: string;
    enabled: boolean;
}
interface ConfirmPasswordRule {
    message?: string;
    enabled: boolean;
    compareToProperty?: string;
}

declare function IsPassword(ValidatorOptions?: PasswordValidatorOptions, defaultValidatorOptions?: ValidationOptions): (object: object, propertyName: string) => void;

declare class PasswordConstraint implements ValidatorConstraintInterface {
    private errorMessage;
    private readonly defaultSpecialChars;
    private readonly defaultBlacklistValues;
    private readonly defaultMessages;
    private readonly defaultOptions;
    validate(value: string, args: ValidationArguments): boolean;
    defaultMessage(): string;
    /**
     * Merge user-provided options with the default options.
     * Ensures that default messages are used when no custom message is provided.
     *
     * @param userOptions The options provided by the user.
     * @returns Merged options with default values for undefined properties.
     */
    private mergeWithDefaults;
    /**
     * Perform all password validations.
     *
     * @param password The password to validate.
     * @param confirmPassword The password confirmation value (if applicable).
     * @param options The validation options to apply.
     * @returns True if the password passes all validations; otherwise, false.
     */
    private validatePassword;
    /**
     * Validate if the password is not in the blacklist.
     */
    private checkBlacklist;
    /**
     * Validate the password length (min and max).
     */
    private checkLength;
    /**
     * Validate if the password matches a specific regex.
     */
    private checkRegex;
    /**
     * Validate if the password contains special characters.
     */
    private checkSpecialChars;
    /**
     * Validate if the confirmation password matches the password.
     */
    private checkConfirmPassword;
}

export { type BlacklistRule, type ConfirmPasswordRule, type HasRule, type HasSpecialCharsRule, IsPassword, type LengthRule, PasswordConstraint, type PasswordValidatorOptions };
